/* cisco-pix.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "plugin.h"
#include "err.h"
#include "hashinterface.h"

int vectorsize;


char * hash_plugin_summary(void)
{
    return("cisco-pix \tCisco PIX password hashes plugin");
}


char * hash_plugin_detailed(void)
{
    return("cisco-pix - Cisco PIX password hashes plugin\n"
	    "------------------------\n"
	    "Use this module to crack Cisco PIX passwords\n"
	    "Input should be in form: \'hash\'\n"
	    "Known software that uses this password hashing method:\n"
	    "Cisco PIX\n"
	    "\nAuthor: Milen Rangelov <gat3way@gat3way.eu>\n");
}


hash_stat hash_plugin_parse_hash(char *hashline, char *filename)
{
    char username[HASHFILE_MAX_LINE_LENGTH];
    char hash[HASHFILE_MAX_LINE_LENGTH];
    char line[HASHFILE_MAX_LINE_LENGTH];
    char *temp_str;
    
    if (!hashline) return hash_err;
    
    if (strlen(hashline)<2) return hash_err;
    

    (void)hash_add_username("PIX enable pwd");

    (void)hash_add_hash(hashline,0);

    (void)hash_add_salt("123");
    (void)hash_add_salt2("");

    return hash_ok;
}


hash_stat hash_plugin_check_hash(const char *hash, char *password[VECTORSIZE], const char *salt,  char * salt2[VECTORSIZE], const char *username, int *num, int threadid)
{
    char *final[VECTORSIZE];
    int cnt,len[VECTORSIZE];
    int a;
    char *salt3;
    
    for (a=0;a<vectorsize;a++) {final[a] = alloca(16);}
    for (a=0;a<vectorsize;a++)
    {
	len[a]=strlen(password[a]);
	for (cnt=len[a];cnt<16;cnt++) password[a][cnt] = 0;
    }
    
    hash_md5(password, final, 16, THREAD_LENPROVIDED);
    
    for (a=0;a<vectorsize;a++)
    {
	salt3 = salt2[a];
	_to64(salt3, *(unsigned long *) (final[a]+0), 4); salt3 += 4;
        _to64(salt3, *(unsigned long *) (final[a]+4), 4); salt3 += 4;
        _to64(salt3, *(unsigned long *) (final[a]+8), 4); salt3 += 4;
        _to64(salt3, *(unsigned long *) (final[a]+12), 4); salt3 += 4;
        *salt3  = '\0';
    }
    for (a=0;a<vectorsize;a++) if (memcmp(salt2[a],hash,strlen(hash))==0) {*num=a;return hash_ok;}
    return hash_err;
}


int hash_plugin_hash_length(void)
{
    return 0;
}

int hash_plugin_is_raw(void)
{
    return 0;
}

int hash_plugin_is_special(void)
{
    return 0;
}

void get_vector_size(int size)
{
    vectorsize = size;
}
