/*
   All copyrights to RAR are exclusively
   owned by the author - Alexander Roshal.
   
   This code may not be used to develop a RAR (WinRAR) compatible archiver.
*/


/* rsa-privkey.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <alloca.h>
#include <fcntl.h>
#include <sys/types.h>
#include <wchar.h>
#include <stdlib.h>
#include "zlib.h"
#include "plugin.h"
#include "err.h"
#include "hashinterface.h"


#define NROUNDS 32
#define NM  1024
#define DC  60
#define NC 299
#define LDC 17
#define SALT_SIZE 8
#define  rol(x,n,xsize)  (((x)<<(n)) | ((x)>>(xsize-(n))))
#define CHECK_BIT(var,pos) ((var) & (1<<(pos)))
#define substLong(t) ( (uint)SubstTable[(uint)t&255] | \
	             ((uint)SubstTable[(int)(t>> 8)&255]<< 8) | \
	             ((uint)SubstTable[(int)(t>>16)&255]<<16) | \
	             ((uint)SubstTable[(int)(t>>24)&255]<<24) )
#define MAXWINSIZE      0x400000
#define MAXWINMASK      (MAXWINSIZE-1)
#define LOW_DIST_REP_COUNT 16
#define RC  28
#define BC  20

#define ARI_DEC_NORMALIZE(code,low,range,read)                           \
{                                                                        \
  while ((low^(low+range))<TOP || range<BOT && ((range=-low&(BOT-1)),1)) \
  {                                                                      \
    code=(code << 8) | read->GetChar();                                  \
    range <<= 8;                                                         \
    low <<= 8;                                                           \
  }                                                                      \
}



static char myfilename[255];
static unsigned int g_CrcTable[256];
static long filepos;
static long packedsize;
static long unpackedsize;
static uint64_t packedsize64;
static uint64_t unpackedsize64;
static long filecrc;

static char salt[8];
static int issalt;
static int islarge;
static unsigned short namesize;
static char encname[256];


struct LitDecode
{
    unsigned int MaxNum;
    unsigned int DecodeLen[16];
    unsigned int DecodePos[16];
    unsigned int DecodeNum[NC];
};
struct LowDistDecode
{
    unsigned int MaxNum;
    unsigned int DecodeLen[16];
    unsigned int DecodePos[16];
    unsigned int DecodeNum[LDC];
};
struct DistDecode
{
    unsigned int MaxNum;
    unsigned int DecodeLen[16];
    unsigned int DecodePos[16];
    unsigned int DecodeNum[DC];
};
struct RepDecode
{
  unsigned int MaxNum;
  unsigned int DecodeLen[16];
  unsigned int DecodePos[16];
  unsigned int DecodeNum[RC];
};
struct BitDecode
{
    unsigned int MaxNum;
    unsigned int DecodeLen[16];
    unsigned int DecodePos[16];
    unsigned int DecodeNum[BC];
};


struct PPM_CONTEXT;
struct STATE;
struct FreqData;

struct STATE
{
  unsigned char Symbol;
  unsigned char Freq;
  struct PPM_CONTEXT* Successor;
};

struct FreqData
{
  unsigned short SummFreq;
  struct STATE * Stats;
};

struct PPM_CONTEXT
{
    unsigned short NumStats;
    union
    {
      struct FreqData U;
      struct STATE OneState;
    };

    struct PPM_CONTEXT* Suffix;
/*    void encodeBinSymbol(ModelPPM *Model,int symbol);  // MaxOrder:
    void encodeSymbol1(ModelPPM *Model,int symbol);    //  ABCD    context
    void encodeSymbol2(ModelPPM *Model,int symbol);    //   BCD    suffix
    void decodeBinSymbol(ModelPPM *Model);  //   BCDE   successor
    int decodeSymbol1(ModelPPM *Model);    // other orders:
    int decodeSymbol2(ModelPPM *Model);    //   BCD    context
    void update1(ModelPPM *Model,STATE* p); //    CD    suffix
    void update2(ModelPPM *Model,STATE* p); //   BCDE   successor
    void rescale(ModelPPM *Model);
    PPM_CONTEXT* createChild(ModelPPM *Model,STATE* pStats,STATE& FirstState);
    SEE2_CONTEXT* makeEscFreq2(ModelPPM *Model,int Diff);*/
};


unsigned int CRCTab[256];
unsigned char PN1,PN2,PN3;
unsigned short OldKey[4];
wchar_t PswW[64];
unsigned char SubstTable[256];
unsigned char AESKey[16],AESInit[16];
unsigned int Key[4];
unsigned int OldDist[4],OldDistPtr;
unsigned int LastDist,LastLength;
unsigned int UnpPtr,WrPtr;
int FileExtracted, Suspended, UnpBlockType;
int InAddr,InBit;
int ReadBorder, PPMEscChar;
uint64_t WrittenFileSize, DestUnpSize;
enum BLOCK_TYPES {BLOCK_LZ,BLOCK_PPM};
struct LitDecode LD;
struct DistDecode DD;
struct LowDistDecode LDD;
struct RepDecode RD;
struct PPM_CONTEXT *MinContext, *MedContext, *MaxContext;
struct BitDecode BD;
const unsigned int TOP=1 << 24, BOT=1 << 15;
unsigned char *Window;
unsigned char Symbol;
int TablesRead;
int PrevLowDist,LowDistRepCount;
unsigned char Ch, NextCh;
unsigned int Length;
int ReadTop;

static unsigned char InitSubstTable[256]={
  215, 19,149, 35, 73,197,192,205,249, 28, 16,119, 48,221,  2, 42,
  232,  1,177,233, 14, 88,219, 25,223,195,244, 90, 87,239,153,137,
  255,199,147, 70, 92, 66,246, 13,216, 40, 62, 29,217,230, 86,  6,
   71, 24,171,196,101,113,218,123, 93, 91,163,178,202, 67, 44,235,
  107,250, 75,234, 49,167,125,211, 83,114,157,144, 32,193,143, 36,
  158,124,247,187, 89,214,141, 47,121,228, 61,130,213,194,174,251,
   97,110, 54,229,115, 57,152, 94,105,243,212, 55,209,245, 63, 11,
  164,200, 31,156, 81,176,227, 21, 76, 99,139,188,127, 17,248, 51,
  207,120,189,210,  8,226, 41, 72,183,203,135,165,166, 60, 98,  7,
  122, 38,155,170, 69,172,252,238, 39,134, 59,128,236, 27,240, 80,
  131,  3, 85,206,145, 79,154,142,159,220,201,133, 74, 64, 20,129,
  224,185,138,103,173,182, 43, 34,254, 82,198,151,231,180, 58, 10,
  118, 26,102, 12, 50,132, 22,191,136,111,162,179, 45,  4,148,108,
  161, 56, 78,126,242,222, 15,175,146, 23, 33,241,181,190, 77,225,
    0, 46,169,186, 68, 95,237, 65, 53,208,253,168,  9, 18,100, 52,
  116,184,160, 96,109, 37, 30,106,140,104,150,  5,204,117,112, 84
};


/* unrar proto functions */
void UpdKeys(unsigned char *Buf);
void Swap(unsigned char *Ch1,unsigned char *Ch2);
void EncryptBlock20(unsigned char *Buf);
unsigned char* WideToRaw(const wchar_t *Src,unsigned char *Dest,int DestSize);
int WideToChar(const wchar_t *Src,char *Dest,int DestSize);
int CharToWide(const char *Src,wchar_t *Dest,int DestSize);
void InitCRC();
unsigned int CRC(unsigned int StartCRC,const void *Addr,size_t Size);
void SetOldKeys(char *Password);
void DecryptBlock(unsigned char *Buf, unsigned char *Buf2, int Size);
void SetCryptKeys(char *Password,unsigned char *Salt, int Encrypt, int OldOnly, int HandsOffHash);







char * hash_plugin_summary(void)
{
    return("rar \t\tRAR3 passwords plugin");
}


char * hash_plugin_detailed(void)
{
    return("rar - RAR3 passwords plugin\n"
	    "------------------------------------------------\n"
	    "Use this module to crack RAR archives passwords\n"
	    "Input should be a passworded RAR file specified with -f\n"
	    "Known software that uses this password hashing method:\n"
	    "WinRAR\n"
	    "\nAuthor: Milen Rangelov <gat3way@gat3way.eu>\n");
}


hash_stat hash_plugin_parse_hash(char *hashline, char *filename)
{
    int fd, cnt;
    char buf[4096];
    unsigned int u321;
    unsigned short u161;
    unsigned char u81;
    char signature[7];
    
    issalt = islarge = 0;
    strcpy(myfilename, filename);

    fd = open(filename, O_RDONLY);
    if (fd<1)
    {
	elog("Cannot open file %s\n", filename);
	return hash_err;
    }
    read(fd,signature,7);
    filepos = 7;

    if ( (signature[0]!=0x52) || (signature[1]!=0x61) || (signature[2]!=0x72) || 
	 (signature[3]!=0x21) || (signature[4]!=0x1a) || (signature[5]!=0x07))
    {
	elog("Not a RAR3 archive: %s", filename);
	return hash_err;
    }
    /* header CRC */
    read(fd, &u161,2);
    filepos+=2;
    /* header type */
    read(fd, &u81,1);
    filepos++;
    hlog("Header type: 0x%02x\n",u81);
    /* read header flags and size so that we can skip to the next header */
    read(fd, &u161,2);
    read(fd, &u161,2);
    filepos+=4;
    hlog("HEADER LENGTH: %d\n", u161);
    read(fd,buf, u161-7);
    filepos+=u161-7;
    
    /* header CRC */
    read(fd, &u161,2);
    filepos+=2;
    /* header type */
    read(fd, &u81,1);
    hlog("Header type: 0x%02x\n",u81);
    read(fd,&u161,2);
    filepos+=3;
    if (!(u161 & 0x4))
    {
	elog("RAR archive %s is not password protected!\n",filename);
	return hash_err;
    }
    if ((u161 & 0x400))
    {
	issalt = 1;
    }
    if ((u161 & 0x100))
    {
	islarge = 1;
    }

    read(fd,&u161,2);
    filepos+=u161-7;
    read(fd,&packedsize,4);
    read(fd,&unpackedsize,4);
    read(fd,&u81,1);
    read(fd,&filecrc,4);
    /* read time,ver,method = 6 bytes */
    read(fd, buf, 6);
    /* read namesize */
    read(fd, &namesize,2);
    printf("namesize=%d\n",namesize);
    /* read attr */
    read(fd, &u321, 4);
    /* read 64-bit size if used */
    if (islarge)
    {
	read(fd,&packedsize64,8);
	read(fd,&unpackedsize64,8);
    }
    read(fd, encname, namesize);
    //printf("encname=%s\n",encname);
    if (issalt)
    {
	read(fd, salt, 8);
	//printf("salt=%s\n",salt);
    }

    close(fd);

    (void)hash_add_username(filename);
    (void)hash_add_hash("RAR file        ",0);
    (void)hash_add_salt(salt);
    (void)hash_add_salt2("                              ");

    return hash_ok;
}




hash_stat hash_plugin_check_hash(const char *hash, const char *password, const char *salt,  char * __restrict salt2, const char *username)
{
    int fd;
    char *buf, *buf2;
    
    if (strlen(password)<1) return hash_err;
    buf = malloc(packedsize);
    buf2 = malloc(packedsize);

    fd = open(myfilename, O_RDONLY);
    if (fd<1)
    {
	elog("Cannot open file %s\n", myfilename);
	return hash_err;
    }
    lseek(fd,filepos,SEEK_SET);
    read(fd, buf, packedsize);
    SetCryptKeys(password, salt, 0, 0, 0);
    DecryptBlock(buf, buf2, packedsize);
    if (CRC(0xffffffff,buf+2,packedsize-2)==filecrc) printf("PASSWORD IS %s!!!\n", password);
    else
    {
	printf("Password: %s, CRCheader=%d, GotCRC=%d\n",password, filecrc,CRC(0xffffffff,buf2+2,packedsize-2));
    }
    free(buf);
    free(buf2);
    
    close(fd);

    return hash_err;
}


int hash_plugin_hash_length(void)
{
    return 16;
}

int hash_plugin_is_raw(void)
{
    return 1;
}

int hash_plugin_is_special(void)
{
    return 1;
}


void UpdKeys(unsigned char *Buf)
{
  int I;
  printf("UpdKeys(%s)\n", Buf);
  for (I=0;I<16;I+=4)
  {
    Key[0]^=CRCTab[Buf[I]];
    Key[1]^=CRCTab[Buf[I+1]];
    Key[2]^=CRCTab[Buf[I+2]];
    Key[3]^=CRCTab[Buf[I+3]];
  }
}



void InitBitInput()
{
	InAddr=InBit=0;
}



void Swap(unsigned char *Ch1,unsigned char *Ch2)
{
    unsigned char Ch=*Ch1;
    *Ch1=*Ch2;
    *Ch2=Ch;
}


void UnpInitData(int Solid)
{
  if (!Solid)
  {
    TablesRead=0;
    memset(OldDist,0,sizeof(OldDist));
    OldDistPtr=0;
    LastDist=LastLength=0;
//    memset(Window,0,MAXWINSIZE);
//    memset(UnpOldTable,0,sizeof(UnpOldTable));
    memset(&LD,0,sizeof(LD));
    memset(&DD,0,sizeof(DD));
    memset(&LDD,0,sizeof(LDD));
    memset(&RD,0,sizeof(RD));
    memset(&BD,0,sizeof(BD));
    UnpPtr=WrPtr=0;
    PPMEscChar=2;
    UnpBlockType=BLOCK_LZ;

    InitFilters();
  }
  InitBitInput();
  WrittenFileSize=0;
  ReadTop=0;
  ReadBorder=0;
  //UnpInitData20(Solid);
}



void EncryptBlock20(unsigned char *Buf)
{
    unsigned int A,B,C,D,T,TA,TB;
    int I;
    
#if defined(BIG_ENDIAN) || !defined(PRESENT_INT32) || !defined(ALLOW_NOT_ALIGNED_INT)
    A=((unsigned int)Buf[0]|((unsigned int)Buf[1]<<8)|((unsigned int)Buf[2]<<16)|((unsigned int)Buf[3]<<24))^Key[0];
    B=((unsigned int)Buf[4]|((unsigned int)Buf[5]<<8)|((unsigned int)Buf[6]<<16)|((unsigned int)Buf[7]<<24))^Key[1];
    C=((unsigned int)Buf[8]|((unsigned int)Buf[9]<<8)|((unsigned int)Buf[10]<<16)|((unsigned int)Buf[11]<<24))^Key[2];
    D=((unsigned int)Buf[12]|((unsigned int)Buf[13]<<8)|((unsigned int)Buf[14]<<16)|((unsigned int)Buf[15]<<24))^Key[3];
#else
    unsigned int *BufPtr=(unsigned int *)Buf;
    A=BufPtr[0]^Key[0];
    B=BufPtr[1]^Key[1];
    C=BufPtr[2]^Key[2];
    D=BufPtr[3]^Key[3];
#endif
    for(I=0;I<NROUNDS;I++)
    {
	T=((C+rol(D,11,32))^Key[I&3]);
        TA=A^substLong(T);
        T=((D^rol(C,17,32))+Key[I&3]);
        TB=B^substLong(T);
        A=C;
        B=D;
        C=TA;
        D=TB;
    }
#if defined(BIG_ENDIAN) || !defined(PRESENT_INT32) || !defined(ALLOW_NOT_ALIGNED_INT)
    C^=Key[0];
    Buf[0]=(unsigned char)C;
    Buf[1]=(unsigned char)(C>>8);
    Buf[2]=(unsigned char)(C>>16);
    Buf[3]=(unsigned char)(C>>24);
    D^=Key[1];
    Buf[4]=(unsigned char)D;
    Buf[5]=(unsigned char)(D>>8);
    Buf[6]=(unsigned char)(D>>16);
    Buf[7]=(unsigned char)(D>>24);
    A^=Key[2];
    Buf[8]=(unsigned char)A;
    Buf[9]=(unsigned char)(A>>8);
    Buf[10]=(unsigned char)(A>>16);
    Buf[11]=(unsigned char)(A>>24);
    B^=Key[3];
    Buf[12]=(unsigned char)B;
    Buf[13]=(unsigned char)(B>>8);
    Buf[14]=(unsigned char)(B>>16);
    Buf[15]=(unsigned char)(B>>24);
#else
    BufPtr[0]=C^Key[0];
    BufPtr[1]=D^Key[1];
    BufPtr[2]=A^Key[2];
    BufPtr[3]=B^Key[3];
#endif
    UpdKeys(Buf);
}


unsigned char* WideToRaw(const wchar_t *Src,unsigned char *Dest,int DestSize)
{
    int I;

    for (I=0;I<DestSize;I++,Src++)
    {
	Dest[I*2]=(unsigned char)*Src;
        Dest[I*2+1]=(unsigned char)(*Src>>8);
        if (*Src==0) break;
    }
    return(Dest);
}



int WideToChar(const wchar_t *Src,char *Dest,int DestSize)
{
    int RetCode=1;
    size_t ResultingSize=wcstombs(Dest,Src,DestSize);
    if (ResultingSize==(size_t)-1)
	RetCode=0;
    if (ResultingSize==0 && *Src!=0)
	RetCode=0;

    if ((!RetCode || (*Dest==0 && *Src!=0)) && DestSize>NM && wcslen(Src)<NM)
    {
	return(WideToChar(Src,Dest,NM));
    }
    return(RetCode);
}


int CharToWide(const char *Src,wchar_t *Dest,int DestSize)
{
  int RetCode=1;
  size_t ResultingSize=mbstowcs(Dest,Src,DestSize);
  if (ResultingSize==(size_t)-1)
    RetCode=0;
  if (ResultingSize==0 && *Src!=0)
    RetCode=0;

  if ( ( (!RetCode) || ((*Dest==0) && (*Src!=0))) && (DestSize>NM) && (strlen(Src)<NM))
  {
    return(CharToWide(Src,Dest,NM));
  }
  return(RetCode);
}

void InitCRC()
{
    int I,J;
    for (I=0;I<256;I++)
    {
	unsigned int C=I;
	for (J=0;J<8;J++)
	C=(C & 1) ? (C>>1)^0xEDB88320L : (C>>1);
	CRCTab[I]=C;
    }
}



unsigned int CRC(unsigned int StartCRC,const void *Addr,size_t Size)
{
    ssize_t I;
    
    if (CRCTab[1]==0)
    InitCRC();
    unsigned char *Data=(unsigned char *)Addr;
#if defined(LITTLE_ENDIAN) && defined(PRESENT_INT32) && defined(ALLOW_NOT_ALIGNED_INT)

    while (Size>0 && ((long)Data & 7))
    {
	StartCRC=CRCTab[(unsigned char)(StartCRC^Data[0])]^(StartCRC>>8);
	Size--;
	Data++;
    }
    while (Size>=8)
    {
	StartCRC^=*(unsigned int *)Data;
	StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
	StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
	StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        StartCRC^=*(unsigned int *)(Data+4);
        StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        StartCRC=CRCTab[(unsigned char)StartCRC]^(StartCRC>>8);
        Data+=8;
        Size-=8;
    }
#endif

    for (I=0;I<Size;I++)
    StartCRC=CRCTab[(unsigned char)(StartCRC^Data[I])]^(StartCRC>>8);
    return(StartCRC);
}









void SetOldKeys(char *Password)
{
    unsigned int PswCRC=CRC(0xffffffff,Password,strlen(Password));
    OldKey[0]=PswCRC&0xffff;
    OldKey[1]=(PswCRC>>16)&0xffff;
    OldKey[2]=OldKey[3]=0;
    PN1=PN2=PN3=0;
    unsigned char Ch; 
    while ((Ch=*Password)!=0)
    {
	PN1+=Ch;
        PN2^=Ch;
        PN3+=Ch;
        PN3=(unsigned char)rol(PN3,1,8);
        OldKey[2]^=Ch^CRCTab[Ch];
        OldKey[3]+=Ch+(CRCTab[Ch]>>16); 
        Password++; 
    } 
} 






void DecryptBlock(unsigned char *Buf, unsigned char *Buf2, int Size)
{
    printf("DecryptBlock(%s,%d)\n", Buf, Size);
//    rin.blockDecrypt(Buf,Size,Buf);
    hash_aes_decrypt(AESKey, 16, Buf, Size, AESInit, Buf2, 1);

}




void SetCryptKeys(char *Password,unsigned char *Salt, int Encrypt, int OldOnly, int HandsOffHash)
{
    int I,J,K;
    int PswLength;
    unsigned int digest[5];    
    
    printf("SetCryptKeys(%s,%s,%d,%d,%d)\n",Password, Salt, Encrypt, OldOnly, HandsOffHash);
    if (*Password==0) return;
    if (OldOnly)
    {
        if (CRCTab[1]==0) InitCRC();
        unsigned char Psw[64];
        SetOldKeys(Password);
        Key[0]=0xD3A3B879L;
        Key[1]=0x3F6D12F7L;
        Key[2]=0x7515A235L;
        Key[3]=0xA4E7F123L;
        memset(Psw,0,sizeof(Psw));
        strcpy((char *)Psw,Password);
        PswLength=strlen(Password);
        memcpy(SubstTable,InitSubstTable,sizeof(SubstTable));
        for (J=0;J<256;J++)
        for (I=0;I<PswLength;I+=2)
        {
	    unsigned int N1=(unsigned char)CRCTab[(Psw[I]-J)&0xff];
	    unsigned int N2=(unsigned char)CRCTab[(Psw[I+1]+J)&0xff];
    	    for (K=1;N1!=N2;N1=(N1+1)&0xff,K++)
    	    Swap(&SubstTable[N1],&SubstTable[(N1+I+K)&0xff]);
	}
	for (I=0;I<PswLength;I+=16) 
	    EncryptBlock20(&Psw[I]); 
	return;
    }

    int Cached=0;
/*    for (I=0;I<sizeof(Cache)/sizeof(Cache[0]);I++)
    if (strcmp(Cache[I].Password,Password)==0 && (Salt==NULL && !Cache[I].SaltPresent || Salt!=NULL &&
	Cache[I].SaltPresent && memcmp(Cache[I].Salt,Salt,SALT_SIZE)==0) &&
	Cache[I].HandsOffHash==HandsOffHash) 
    {
	memcpy(AESKey,Cache[I].AESKey,sizeof(AESKey));   
	memcpy(AESInit,Cache[I].AESInit,sizeof(AESInit));
	Cached = 1;
	break;
    }*/

    if (!Cached)
    { 
	wchar_t PswW[64];
	CharToWide(Password,PswW,64-1);
	PswW[64-1]=0;
	unsigned char RawPsw[2*64+SALT_SIZE];
	WideToRaw(PswW, RawPsw, 0x1000000);
	int RawLength=2*wcslen(PswW);
	if (Salt!=NULL)
	{
	    memcpy(RawPsw+RawLength,Salt,SALT_SIZE);
	    RawLength+=SALT_SIZE;
        }
//	hash_context c;
//	hash_initial(&c);
	char *hashbuf=alloca(65535);
	int hashpos=0;

	const int HashRounds=0x40000;
	for (I=0;I<HashRounds;I++)
	{
//	    hash_process( &c, RawPsw, RawLength, HandsOffHash);
	    memcpy(hashbuf+hashpos, RawPsw, RawLength);
	    hashpos+=RawLength;

	    unsigned char PswNum[3];
	    PswNum[0]=(unsigned char)I;
	    PswNum[1]=(unsigned char)(I>>8);
	    PswNum[2]=(unsigned char)(I>>16);
//	    hash_process( &c, PswNum, 3, HandsOffHash);
	    memcpy(hashbuf+hashpos, PswNum, 3);
	    hashpos+=3;


	    if (I%(HashRounds/16)==0) 
	    {
//		hash_context tempc=c;
//		hash_final( &tempc, digest, HandsOffHash);
		hash_sha1(hashbuf, (char *)&digest, hashpos);
		//hashpos = 0;

		AESInit[I/(HashRounds/16)]=(unsigned char)digest[4];
	    }
	}
//	hash_final( &c, digest, HandsOffHash);
	hash_sha1(hashbuf, (char *)&digest, hashpos);
	//hashpos = 0;

	for (I=0;I<4;I++)  
	for (J=0;J<4;J++)
	    AESKey[I*4+J]=(unsigned char)(digest[I]>>(J*8));

/*	strcpy(Cache[CachePos].Password,Password);
	if ((Cache[CachePos].SaltPresent=(Salt!=NULL))==1)
	    memcpy(Cache[CachePos].Salt,Salt,SALT_SIZE);  
	Cache[CachePos].HandsOffHash=HandsOffHash; 
	memcpy(Cache[CachePos].AESKey,AESKey,sizeof(AESKey));
	memcpy(Cache[CachePos].AESInit,AESInit,sizeof(AESInit));
	CachePos=(CachePos+1)%(sizeof(Cache)/sizeof(Cache[0])); */
    }
//    rin.init(Encrypt ? Rijndael::Encrypt : Rijndael::Decrypt,AESKey,AESInit);
}




inline void InsertOldDist(unsigned int Distance)
{
  OldDist[3]=OldDist[2];
  OldDist[2]=OldDist[1];
  OldDist[1]=OldDist[0];
  OldDist[0]=Distance;
}


inline void InsertLastMatch(unsigned int Length,unsigned int Distance)
{
  LastDist=Distance;
  LastLength=Length;
}


void CopyString(unsigned int Length,unsigned int Distance)
{
  unsigned int DestPtr=UnpPtr-Distance;
  if (DestPtr<MAXWINSIZE-260 && UnpPtr<MAXWINSIZE-260)
  {
    Window[UnpPtr++]=Window[DestPtr++];
    while (--Length>0)
      Window[UnpPtr++]=Window[DestPtr++];
  }
  else
    while (Length--)
    {
      Window[UnpPtr]=Window[DestPtr++ & MAXWINMASK];
      UnpPtr=(UnpPtr+1) & MAXWINMASK;
    }
}


int DecodeChar()
{
/*  if ((unsigned char*)MinContext <= SubAlloc.pText || (unsigned char*)MinContext>SubAlloc.HeapEnd)
    return(-1);
  if (MinContext->NumStats != 1)
  {
    if ((unsigned char*)MinContext->U.Stats <= SubAlloc.pText || (unsigned char*)MinContext->U.Stats>SubAlloc.HeapEnd)
      return(-1);
    if (!MinContext->decodeSymbol1(this))
      return(-1);
  }
  else
    MinContext->decodeBinSymbol(this);
  Coder.Decode();
  while ( !FoundState )
  {
    ARI_DEC_NORMALIZE(Coder.code,Coder.low,Coder.range,Coder.UnpackRead);
    do
    {
      OrderFall++;
      MinContext=MinContext->Suffix;
      if ((unsigned char*)MinContext <= SubAlloc.pText || (unsigned char*)MinContext>SubAlloc.HeapEnd)
        return(-1);
    } while (MinContext->NumStats == NumMasked);
    if (!MinContext->decodeSymbol2(this))
      return(-1);
    Coder.Decode();
  }
  int Symbol=FoundState->Symbol;
  if (!OrderFall && (unsigned char*) FoundState->Successor > SubAlloc.pText)
    MinContext=MaxContext=FoundState->Successor;
  else
  {
    UpdateModel();
    if (EscCount == 0)
      ClearMask();
  }
  ARI_DEC_NORMALIZE(Coder.code,Coder.low,Coder.range,Coder.UnpackRead);*/
  return(Symbol);
}




void Unpack29(int Solid)
{
  static unsigned char LDecode[]={0,1,2,3,4,5,6,7,8,10,12,14,16,20,24,28,32,40,48,56,64,80,96,112,128,160,192,224};
  static unsigned char LBits[]=  {0,0,0,0,0,0,0,0,1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4,  4,  5,  5,  5,  5};
  static int DDecode[DC];
  static unsigned char DBits[DC];
  static int DBitLengthCounts[]= {4,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,14,0,12};
  static unsigned char SDDecode[]={0,4,8,16,32,64,128,192};
  static unsigned char SDBits[]=  {2,2,3, 4, 5, 6,  6,  6};
  unsigned int Bits;
  int I,J;

  if (DDecode[1]==0)
  {
    int Dist=0,BitLength=0,Slot=0;
    for (I=0;I<sizeof(DBitLengthCounts)/sizeof(DBitLengthCounts[0]);I++,BitLength++)
      for (J=0;J<DBitLengthCounts[I];J++,Slot++,Dist+=(1<<BitLength))
      {
        DDecode[Slot]=Dist;
        DBits[Slot]=BitLength;
      }
  }

  FileExtracted=1;

  if (!Suspended)
  {
    UnpInitData(Solid);
    if (!UnpReadBuf())
      return;
    if ((!Solid || !TablesRead) && !ReadTables())
      return;
  }

  while (1)
  {
    UnpPtr&=MAXWINMASK;

    if (InAddr>ReadBorder)
    {
      if (!UnpReadBuf())
        break;
    }
    if (((WrPtr-UnpPtr) & MAXWINMASK)<260 && WrPtr!=UnpPtr)
    {
      UnpWriteBuf();
      if (WrittenFileSize>DestUnpSize)
        return;
      if (Suspended)
      {
        FileExtracted=0;
        return;
      }
    }
    if (UnpBlockType==BLOCK_PPM)
    {
      //int Ch=PPM.DecodeChar();
      if (Ch==-1)
      {
        //PPM.CleanUp();

        // turn off PPM compression mode in case of error, so UnRAR will
        // call PPM.DecodeInit in case it needs to turn it on back later.
        UnpBlockType=BLOCK_LZ;
        break;
      }
      if (Ch==PPMEscChar)
      {
        //int NextCh=PPM.DecodeChar();
        if (NextCh==0)
        {
          if (!ReadTables())
            break;
          continue;
        }
        if (NextCh==2 || NextCh==-1)
          break;
        if (NextCh==3)
       {
          if (!ReadVMCodePPM())
            break;
          continue;
        }
        if (NextCh==4)
        {
          unsigned int Distance=0,Length;
          int Failed=0;
          for (I=0;I<4 && !Failed;I++)
          {
//            int Ch=PPM.DecodeChar();
            if (Ch==-1)
              Failed=1;
            else
              if (I==3)
                Length=(unsigned char)Ch;
              else
                Distance=(Distance<<8)+(unsigned char)Ch;
          }
          if (Failed)
            break;

          CopyString(Length+32,Distance+2);
          continue;
        }
        if (NextCh==5)
        {
          //int Length=PPM.DecodeChar();
          if (Length==-1)
            break;
          CopyString(Length+4,1);
          continue;
        }
     }
      Window[UnpPtr++]=Ch;
      continue;
    }

    int Number=DecodeNumber((struct Decode *)&LD);
    if (Number<256)
    {
      Window[UnpPtr++]=(unsigned char)Number;
      continue;
    }
    if (Number>=271)
    {
      int Length=LDecode[Number-=271]+3;
      if ((Bits=LBits[Number])>0)
      {
        Length+=getbits()>>(16-Bits);
        addbits(Bits);
      }

      int DistNumber=DecodeNumber((struct Decode *)&DD);
      unsigned int Distance=DDecode[DistNumber]+1;
      if ((Bits=DBits[DistNumber])>0)
      {
        if (DistNumber>9)
        {
          if (Bits>4)
          {
            Distance+=((getbits()>>(20-Bits))<<4);
            addbits(Bits-4);
          }
          if (LowDistRepCount>0)
          {
            LowDistRepCount--;
            Distance+=PrevLowDist;
          }
          else
          {
            int LowDist=DecodeNumber((struct Decode *)&LDD);
            if (LowDist==16)
            {
              LowDistRepCount=LOW_DIST_REP_COUNT-1;
              Distance+=PrevLowDist;
            }
            else
            {
              Distance+=LowDist;
              PrevLowDist=LowDist;
            }
          }
        }
        else
        {
          Distance+=getbits()>>(16-Bits);
          addbits(Bits);
        }
      }

      if (Distance>=0x2000)
      {
        Length++;
        if (Distance>=0x40000L)
          Length++;
      }

      InsertOldDist(Distance);
      InsertLastMatch(Length,Distance);
      CopyString(Length,Distance);
      continue;
    }
    if (Number==256)
    {
      if (!ReadEndOfBlock())
        break;
      continue;
    }
    if (Number==257)
    {
      if (!ReadVMCode())
        break;
      continue;
    }
    if (Number==258)
    {
      if (LastLength!=0)
        CopyString(LastLength,LastDist);
      continue;
    }
    if (Number<263)
    {
      int DistNum=Number-259;
      unsigned int Distance=OldDist[DistNum];
      for (I=DistNum;I>0;I--)
        OldDist[I]=OldDist[I-1];
      OldDist[0]=Distance;

      int LengthNumber=DecodeNumber((struct Decode *)&RD);
      int Length=LDecode[LengthNumber]+2;
      if ((Bits=LBits[LengthNumber])>0)
      {
        Length+=getbits()>>(16-Bits);
        addbits(Bits);
      }
      InsertLastMatch(Length,Distance);
      CopyString(Length,Distance);
      continue;
    }
    if (Number<272)
    {
      unsigned int Distance=SDDecode[Number-=263]+1;
      if ((Bits=SDBits[Number])>0)
      {
        Distance+=getbits()>>(16-Bits);
        addbits(Bits);
      }
      InsertOldDist(Distance);
      InsertLastMatch(2,Distance);
      CopyString(2,Distance);
      continue;
    }
  }
  UnpWriteBuf();
}


void DoUnpack(int Method,int Solid)
{
  switch(Method)
  {
/*
    case 15: // rar 1.5 compression
      Unpack15(Solid);
      break;
    case 20: // rar 2.x compression
    case 26: // files larger than 2GB
      Unpack20(Solid);
      break;
*/
    case 29: // rar 3.x compression
    case 36: // alternative hash
      Unpack29(Solid);
      break;
  }
}
