/* threads.h
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef HASHTHREADS_H
#define HASHTHREADS_H

#include <pthread.h>
#include <semaphore.h>
#include <stdint.h>
#include "hashinterface.h"
#include "err.h"


/* global variables */
pthread_t hash_threads[HASHKILL_MAXTHREADS];
unsigned int hash_thread_queue_counts[HASHKILL_MAXTHREADS][32];
unsigned int hash_thread_processed[HASHKILL_MAXTHREADS][32];
unsigned int hash_thread_cracked[HASHKILL_MAXTHREADS];
unsigned int hash_num_threads;
unsigned int hash_queue_size;

/* mutexes - public since list routines need them */
pthread_mutex_t listmutex;
pthread_mutex_t crackedmutex;
pthread_mutex_t queuelock[HASHKILL_MAXTHREADS];


/* Thread functions */
hash_stat spawn_threads(unsigned int num, unsigned int fifo_size);
unsigned int hash_num_cpu(void);
unsigned long hash_get_total_memory(void); 
hash_stat init_mutexes(void);
hash_stat main_thread_dictionary(void);
hash_stat main_thread_bruteforce(void);
hash_stat main_thread_hybrid(void);


/* Helper get/set routines */
void thread_set_bf_len(int len);
void thread_set_bf_str(char *str);
void thread_set_dict_pos(long len);
void thread_set_bf_pos(uint64_t pos);
void thread_set_markov_pos(uint64_t pos);
void thread_set_markov_max_pos(uint64_t pos);
void thread_set_markov_str(char *str);
void thread_set_markov_len(int len);

/* Cleanup routine */
void thread_attack_cleanup(void);

#endif
